@@ -1,6 +1,6 @@
-/*	$Id: kcgi.h,v 1.99 2017/09/15 20:36:03 kristaps Exp $ */
+/*	$Id: kcgi.h,v 1.114 2018/03/06 04:08:36 kristaps Exp $ */
 /*
- * Copyright (c) 2012, 2014--2017 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) 2012, 2014--2018 Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -32,6 +32,44 @@
 #  endif
 #endif
 
+/*
+ * Stringification of version major, minor, and build.
+ */
+#define NAME(s) NAME0(s)
+#define NAME0(s) #s
+#define NAME2(x,y,z) x ## . ## y ## . ## z
+#define NAME1(x,y,z) NAME2(x,y,z)
+
+/*
+ * Major version.
+ */
+#define	KCGI_VMAJOR	0
+
+/*
+ * Minor version.
+ */
+#define	KCGI_VMINOR	10
+
+/*
+ * Build version.
+ */
+#define	KCGI_VBUILD	1
+
+/*
+ * Version string of major.minor.build (as a literal string).
+ */
+#define	KCGI_VERSION	NAME(NAME1(KCGI_VMAJOR,KCGI_VMINOR,KCGI_VBUILD))
+
+/*
+ * Integral stamp of version.
+ * Guaranteed to be increasing with build, minor, and major.
+ * (Assumes build and minor never go over 100.)
+ */
+#define	KCGI_VSTAMP \
+	((KCGI_VBUILD+1) + \
+	 (KCGI_VMINOR+1)*100 + \
+	 (KCGI_VMAJOR+1)*10000)
+
 /*
  * All of the public functions, variables, and structures in this header
  * file are described in kcgi(3).
@@ -286,6 +324,7 @@ enum	kmime {
 	KMIME_APP_JSON,
 	KMIME_APP_OCTET_STREAM,
 	KMIME_APP_PDF,
+	KMIME_APP_ZIP,
 	KMIME_IMAGE_GIF,
 	KMIME_IMAGE_JPEG,
 	KMIME_IMAGE_PNG,
@@ -369,7 +408,7 @@ struct	khttpdigest {
 	char		*nonce;
 	char		*cnonce;
 	char		*response;
-	size_t		 count;
+	uint32_t	 count;
 	char		*opaque;
 };
 
@@ -430,6 +469,13 @@ struct	kopts {
 	ssize_t		  	  sndbufsz;
 };
 
+struct	kcgi_buf {
+	char		*buf; /* buffer contents */
+	size_t		 maxsz; /* buffer size (allocated) */
+	size_t		 sz; /* buffer current length */
+	size_t		 growsz; /* amount to grow (0 == default) */
+};
+
 struct	ktemplate {
 	const char *const	 *key;
 	size_t		 	  keysz;
@@ -437,7 +483,7 @@ struct	ktemplate {
 	int		 	(*cb)(size_t, void *);
 };
 
-typedef int (*ktemplate_writef)(const char *, size_t, void *);
+typedef enum kcgi_err (*ktemplate_writef)(const char *, size_t, void *);
 
 struct	ktemplatex {
 	ktemplate_writef	 writer;
@@ -462,11 +508,11 @@ enum	kcgi_err {
 
 __BEGIN_DECLS
 
-int		 khttp_body(struct kreq *);
-int		 khttp_body_compress(struct kreq *, int);
+enum kcgi_err	 khttp_body(struct kreq *);
+enum kcgi_err	 khttp_body_compress(struct kreq *, int);
 void		 khttp_free(struct kreq *);
 void		 khttp_child_free(struct kreq *);
-void		 khttp_head(struct kreq *, const char *, 
+enum kcgi_err	 khttp_head(struct kreq *, const char *, 
 			const char *, ...) 
 			__attribute__((format(printf, 3, 4)));
 enum kcgi_err	 khttp_parse(struct kreq *, 
@@ -478,26 +524,30 @@ enum kcgi_err	 khttp_parsex(struct kreq *, const struct kmimemap *,
 			const char *const *, size_t,
 			size_t, size_t, void *, void (*)(void *),
 			unsigned int, const struct kopts *);
-void		 khttp_putc(struct kreq *, int);
-void		 khttp_puts(struct kreq *, const char *);
-int		 khttp_template(struct kreq *, 
+enum kcgi_err	 khttp_putc(struct kreq *, int);
+enum kcgi_err	 khttp_puts(struct kreq *, const char *);
+enum kcgi_err	 khttp_template(struct kreq *, 
 			const struct ktemplate *, const char *);
-int		 khttp_template_fd(struct kreq *, 
+enum kcgi_err	 khttp_template_fd(struct kreq *, 
 			const struct ktemplate *, int, const char *);
-int		 khttp_template_buf(struct kreq *, 
+enum kcgi_err	 khttp_template_buf(struct kreq *, 
 			const struct ktemplate *, const char *, 
 			size_t);
-int		 khttp_templatex(const struct ktemplate *, 
+enum kcgi_err	 khttp_templatex(const struct ktemplate *, 
 			const char *, const struct ktemplatex *, 
 			void *);
-int		 khttp_templatex_buf(const struct ktemplate *, 
+enum kcgi_err	 khttp_templatex_buf(const struct ktemplate *, 
 			const char *, size_t, 
 			const struct ktemplatex *, void *);
-int		 khttp_templatex_fd(const struct ktemplate *, 
+enum kcgi_err	 khttp_templatex_fd(const struct ktemplate *, 
 			int, const char *,
 			const struct ktemplatex *, void *);
 
-void		 khttp_write(struct kreq *, const char *, size_t);
+enum kcgi_err	 khttp_write(struct kreq *, const char *, size_t);
+
+enum kcgi_err	 kcgi_buf_write(const char *, size_t, void *);
+enum kcgi_err	 kcgi_buf_putc(struct kcgi_buf *, char);
+enum kcgi_err	 kcgi_buf_puts(struct kcgi_buf *, const char *);
 
 int		 khttpdigest_validate(const struct kreq *, 
 			const char *);
@@ -506,6 +556,7 @@ int		 khttpdigest_validatehash(const struct kreq *,
 int		 khttpbasic_validate(const struct kreq *, 
 			const char *, const char *);
 
+int		 kvalid_bit(struct kpair *);
 int		 kvalid_date(struct kpair *);
 int		 kvalid_double(struct kpair *);
 int		 kvalid_email(struct kpair *);
@@ -515,6 +566,8 @@ int		 kvalid_stringne(struct kpair *);
 int		 kvalid_udouble(struct kpair *);
 int		 kvalid_uint(struct kpair *);
 
+void		 kcgi_writer_disable(struct kreq *);
+
 enum kcgi_err	 khttp_fcgi_parse(struct kfcgi *, struct kreq *);
 enum kcgi_err	 khttp_fcgi_init(struct kfcgi **, 
 			const struct kvalid *, size_t,
@@ -545,8 +598,11 @@ void		 kutil_epoch2tmvals(int64_t, int *, int *, int *,
 char		*kutil_epoch2str(int64_t, char *, size_t);
 char		*kutil_epoch2utcstr(int64_t, char *, size_t);
 int64_t	 	 kutil_date2epoch(int64_t, int64_t, int64_t);
+int	 	 kutil_date_check(int64_t, int64_t, int64_t);
 int64_t	 	 kutil_datetime2epoch(int64_t, int64_t, int64_t,
 			int64_t, int64_t, int64_t);
+int	 	 kutil_datetime_check(int64_t, int64_t, int64_t,
+			int64_t, int64_t, int64_t);
 
 char		*kutil_urlabs(enum kscheme, const char *, 
 			uint16_t, const char *);
@@ -558,6 +614,12 @@ char		*kutil_urlencode(const char *);
 void		 kutil_invalidate(struct kreq *, struct kpair *);
 
 int		 kutil_openlog(const char *);
+void	 	 kutil_verr(const struct kreq *, 
+			const char *, const char *, va_list)
+			__attribute__((noreturn));
+void	 	 kutil_verrx(const struct kreq *, 
+			const char *, const char *, va_list)
+			__attribute__((noreturn));
 void		 kutil_vinfo(const struct kreq *, 
 			const char *, const char *, va_list);
 void		 kutil_vlog(const struct kreq *, const char *,
@@ -583,6 +645,14 @@ void		 kutil_warn(const struct kreq *,
 void		 kutil_warnx(const struct kreq *, 
 			const char *, const char *, ...)
 			__attribute__((format(printf, 3, 4)));
+void		 kutil_err(const struct kreq *, 
+			const char *, const char *, ...)
+			__attribute__((format(printf, 3, 4)))
+			__attribute__((noreturn));
+void		 kutil_errx(const struct kreq *, 
+			const char *, const char *, ...)
+			__attribute__((format(printf, 3, 4)))
+			__attribute__((noreturn));
 
 int		 kasprintf(char **, const char *, ...)
 			__attribute__((format(printf, 2, 3)));
